class Bordle
  class Dictionary
    DICTIONARY_PATH = Path.new("/usr/share/dict/")

    property length : UInt8
    property data : Array(String)
    property language : String = "french"

    def initialize(length : UInt8)
      @length = length
      @data = [] of String
      load_data
    end

    def load_data
      dictionary_file = DICTIONARY_PATH / @language
      # use french dictionary from wfrench package
      if !File.exists? dictionary_file
        STDERR.puts "ERROR: dictionary file missing! (#{dictionary_file})"
        STDERR.puts "       Please install then w#{@language} package on your system"
        exit 1
      end

      lines = File.read_lines(dictionary_file)
      @data =
        lines
          .select { |word| word.size == @length }
          .map { |word| word.tr(TR_DIACRITICS, TR_ASCII) }
    end

    def includes?(word)
      @data.includes? word
    end

    def choose_word
      @data
        .sample(1)
        .first
    end
  end
end
